##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) 2000, 2001, 2002 Control.com, Inc.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
from DateTime import DateTime
from Products.CMFCore.utils import SimpleItemWithProperties
import Globals
from AccessControl import ClassSecurityInfo
from Products.CMFCore import CMFCorePermissions
from Globals import DTMLFile
from zLOG import LOG, ERROR, INFO

class PostingFilter(SimpleItemWithProperties):
    """
    Control.com message filtering support.
    """

    meta_type = 'SqSQL Posting Filter'

    manage_options = (({'label':'Edit', 'action':'manage_editFilterPropertiesForm'},) +
                      SimpleItemWithProperties.manage_options[:])

    security = ClassSecurityInfo()

    _properties = ({'id':'max_age', 'type':'int', 'mode':'w',
                    'label':'Maximum posting age'},
                   {'id':'keywords', 'type':'lines', 'mode':'w',
                    'label':'Match keywords'},
                   {'id':'syndicated_to', 'type':'lines', 'mode':'w',
                    'label':'Syndicate to'},
                   {'id':'min_items', 'type':'int', 'mode':'w',
                    'label':'Minimum items to return'},
                   {'id':'max_items', 'type':'int', 'mode':'w',
                    'label':'Maximum items to return'},
                   {'id':'sort_on', 'type':'string', 'mode':'w',
                    'label':'Sort by'},
                   {'id':'sort_reverse', 'type':'boolean', 'mode':'w',
                    'label':'Descending sort?'},
                   )

    max_age = 1
    keywords = []
    syndicated_to = []
    max_items = 30
    min_items = 5

    def buildQuery(self, today=DateTime(), reviewed_only=1):
        """
        Compute a query dictionary from the filter.
        
        A number of dtml methods use this to obtain a query dictionary,
        mutate it, then pass the modified dictionary with:

          postings_tool.applyFilter(self, query_dict=dict)

        (Squishdot dtml methods also do similar things.)
        
        """
        query = {}
        query['meta_type'] = 'Article'
        query['reviewed'] = reviewed_only
        if hasattr(self, 'max_age') and self.max_age:
            query['date'] = today - self.max_age
            query['date_usage'] = 'range:min'
        if hasattr(self, 'syndicated_to') and self.syndicated_to:
            query['syndicated_to'] = self.syndicated_to
        elif hasattr(self, 'keywords') and self.keywords:
            query['keywords'] = self.keywords
        if hasattr(self, 'sort_on') and self.sort_on:
            query['sort_on'] = self.sort_on
        if hasattr(self, 'sort_reverse') and self.sort_reverse:
            query['sort_order'] = 'descending'
        return query

    def manage_editFilterProperties(self, REQUEST):
        """
        Change the filter properties.
        """
        self.max_age = REQUEST.get('max_age', 1)
        self.keywords = REQUEST.get('keywords', [])
        self.syndicated_to = REQUEST.get('syndicated_to', [])
        self.max_items = REQUEST.get('max_items', 30)
        self.min_items = REQUEST.get('min_items', 5)
        self.sort_on = REQUEST.get('sort_on', 'date')
        self.sort_reverse = REQUEST.get('sort_reverse', 0)
        if REQUEST is not None:
            return self.manage_editFilterPropertiesForm(
                REQUEST, manage_tabs_message='Changed.')

    manage_editFilterPropertiesForm = DTMLFile('dtml/editPostingFilter', globals())

    #
    # The remaining methods are legacy support for using the same posting filters
    # with Squishdot and SqSQL.
    #

    def filterDups(self, results):
	"""
        Return a result list with all duplicate ids removed.
  	The list does not have to be sorted by id.

        This method was written to hide some Squishdot index bugs that
        resulted in duplicate ids being returnd from searchResults.  It
        really shouldn't be necessary but we wound up moving on to SqSQL
        before we found the bugs.
        """
	found = {}
	new_results = []
	for r in results:
            # It would be nice to check if r is an instance of Squishdot.Posting.Posting
            # here, but that would require doing an r.getObject.
            if found.has_key(r.id):
		LOG('Filters', ERROR, 'Duplicate found for id %s' % str(r.id))
            else:
		found[r.id] = 1
		new_results.append(r)
	return new_results

    def full_apply(self, catalog, today=DateTime()):
        """
        Apply the filter to catalog and return a tuple
        of the Lazy sequence of all results and
        the query dictionary finally used to get them.
        """
        query = self.buildQuery(today)
        results = self.filterDups(catalog.searchResults(query))
        #from zLOG import LOG, INFO
        for i in range(1, 30):
            if len(results) >= self.min_items:
                break
            query['date'] = query['date'] - 1
            #LOG('Filters', INFO, 'Not enough postings (min %d) - increasing max age.' % \
            #    self.min_items)
            results = self.filterDups(catalog.searchResults(query))
        return (results, query)

    def apply(self, catalog, today=DateTime()):
        """
        Apply the filter to catalog and return a Lazy sequence of results.
        """
        results, query = self.full_apply(catalog, today)
        if self.max_items and results:
            if len(results) > self.max_items:
                results = results[:self.max_items]
        return results

    def apply_for_overflow(self, catalog, today=DateTime()):
        """
        Apply the filter to catalog and return a Lazy sequence of overflow results.
        """
        results, query = self.full_apply(catalog, today)
        if self.max_items and results:
            if len(results) > self.max_items:
                # Yes, this should be just, [self.max_items:], but the __getslice__
                # method for Lazy ZCatalog sequences does a
                # range(self.max_items, sys.maxint) if I default the end of the slice!
                return results[self.max_items:len(results)]
        return results[0:0]


Globals.InitializeClass(PostingFilter)

manage_addPostingFilterForm = DTMLFile('dtml/addPostingFilter', globals())
manage_addPostingFilterForm.__name__ = 'addPostingFilter'

def manage_addPostingFilter(self, id, REQUEST=None):
    ' '
    ob = PostingFilter()
    ob.id = id
    self._setObject(id, ob)
    if REQUEST is not None:
        return self.manage_main(self, REQUEST)
