##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) 2000, 2001, 2002 Control.com, Inc.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

__doc__="""CMF tool to maintain Squishdot-like postings in a Postgresql database.

$Id: """
__version__='$Revision: 1.8.2.32 $'[11:-2]

from Products.CMFCore.utils import UniqueObject, _getAuthenticatedUser, _checkPermission
from Products.CMFCore.utils import getToolByName, _dtmldir
from Products.CMFCore.CMFCorePermissions import AccessContentsInformation, ManagePortal
from Globals import InitializeClass, DTMLFile, package_home, PersistentMapping
from AccessControl import ClassSecurityInfo
from string import join, lower
from DateTime import DateTime
from OFS.Folder import Folder
from Products.ZSQLMethods.SQL import SQL
from zLOG import LOG, ERROR, INFO
from time import time
from urllib import quote, unquote     

class Stack:
    def __init__(self):
        self.mlist = []
    def pop(self):
        data = self.mlist[-1]
        del self.mlist[-1]
        return data
    def push(self,data):
        self.mlist.append(data)
    def isEmpty(self):
        return (len(self.mlist) == 0)

class PostingsTool(UniqueObject, Folder):
    """
        Maintain Squishdot style postings in a Postgresql database.
        
        Since there is no database connection passed at tool creation, the
        first manual action with the tool must be to set a database connection
        via the SQL tab.  All other tabs and methods are likely to fail or be
        useless until that is done.
    """
    id = 'postings_tool'
    meta_type = 'SqSQL Postings Tool'

    security = ClassSecurityInfo()

    manage_options = ( {'label':'Moderation',
                        'action':'manage_moderationForm'},
                       {'label':'Postings',
                        'action':'manage_postingsForm'},
                       {'label':'Keywords',
                        'action':'manage_keywordsForm'},
                       {'label':'SQL',
                        'action':'manage_propertiesForm'},
                       {'label':'Debug',
                        'action':'manage_advancedForm'},
                     ) + Folder.manage_options + (
                       { 'label' : 'Help',
                         'action' : 'manage_overview' },)

    def __init__(self):
        self.attachments = PersistentMapping()
        self.connection = None

    security.declareProtected(AccessContentsInformation, 'getConnection')
    def getConnection(self):
        """ Returns the database connection. """
        return self.connection

    #
    #   Zope Management Interface methods
    #
    security.declareProtected(ManagePortal, 'manage_overview')
    manage_overview = DTMLFile('dtml/explainPostingsTool', globals())

    security.declareProtected(ManagePortal, 'manage_moderationForm')
    manage_moderationForm = DTMLFile('dtml/manageModeration', globals())

    security.declareProtected(ManagePortal, 'manage_postingsForm')
    manage_postingsForm = DTMLFile('dtml/managePostings', globals())

    security.declareProtected(ManagePortal, 'manage_keywordsForm')
    manage_keywordsForm = DTMLFile('dtml/manageKeywords', globals())

    security.declareProtected(ManagePortal, 'manage_categoryForm')
    manage_categoryForm = DTMLFile('dtml/manageCategory', globals())

    security.declareProtected(ManagePortal, 'manage_propertiesForm')
    manage_propertiesForm = DTMLFile('dtml/postingsToolProps', globals())

    security.declareProtected(ManagePortal, 'manage_properties')
    def manage_properties(self, connection, REQUEST=None):
        """ Changes postings_tool properties. """

        self.connection = connection
        self.delSQLQueries()
        self.addSQLQueries()
        if REQUEST is not None:
            return self.manage_propertiesForm(
                self, REQUEST,
                manage_tabs_message='Connection changed, ZSQL methods redefined.')

    security.declareProtected(ManagePortal, 'manage_advancedForm')
    manage_advancedForm = DTMLFile('dtml/postingsToolAdvanced', globals())

    security.declareProtected(ManagePortal, 'manage_advanced')
    def manage_advanced(self, REQUEST=None):
        """ Currently just imports all the posts from the control_com Squishdot site. """
        self.fillFromSquishdot(self.control_com)
        if REQUEST is not None:
            return self.manage_advancedForm(
                self, REQUEST, manage_tabs_message='control_com postings imported.')

    #
    #   'postings_tool' interface methods
    #
    security.declareProtected(ManagePortal, 'newPosting')
    def newPosting(self, title, body, subject, parent_id=0, reviewed=0, notify=0,
                   encoding='Plain', member=None, author=None, email=None, summary=None,
                   dept=None, attachment=None, keywords=[], clients=[]):
        """Add a new posting to the database and return it's id."""
        id = self.sqlGetNewId()[0].nextval
        if parent_id:
            parent = self.getFullPosting(parent_id)
            root = parent.root
            level = parent.level + 1
            article = 0
        else:
            parent_id = 0
            root = 0
            level = 0
            article = 1
        created = DateTime()
        modified = DateTime()
        self.addPosting(id, title, body, subject, article=article,
                        parent=parent_id, root=root, level=level, encoding=encoding,
                        created=created, modified=modified,
                        reviewed=reviewed, notify=notify,
                        member=member, author=author, email=email,
                        summary=summary, dept=dept, attachment=attachment,
                        keywords=keywords, clients=clients)
        return id

    security.declareProtected(ManagePortal, 'addPosting')
    def addPosting(self, id, title, body, subject, created=None, modified=None, reviewed=0,
                   notify=0, encoding='Plain', article=0, member=None, author=None,
                   email=None, summary=None, dept=None, parent=None, root=None,
                   level=0, reply_cnt=0, attachment=None, keywords=[], clients=[]):
        """Add an article or reply to the database."""

        # Argument validation
        if not created: created = id
        if not modified: modified = created
        if article:
            root = id
            parent = None
            level = 0
            article = 1
        else:
            assert root
            assert parent
            assert level
            article = 0
        if member:
            author = None
            email = None
        else:
            if not author:
                LOG('SqSQL', ERROR, 'addPosting: id %s no author name' % `id`)
                author = "Anonymous"
        assert encoding in ['Plain', 'HTML', 'STX']
        if not title:
            LOG('SqSQL', ERROR, 'addPosting: id %s empty title' % `id`)
            title = "No Title"
        assert subject                  # TODO: check for valid subject

        # Convert arguments
        reviewed = reviewed and 1 or 0
        notify = notify and 1 or 0

        if type(created) in [type(''), type(0)]:
            created = DateTime(created)
        created = created.aCommon()
        if type(modified) in [type(''), type(0)]:
            modified = DateTime(modified)
        modified = modified.aCommon()

        # attachment is a file object with attributes:
        #   _name : file name string
        #   _file : the data
        #   _created : raw time value
        #   _modified : raw time value
        #   _ctype : mime type string
        # For now, store attachments in a PersistentMapping named attachments
        if self._has_attachment(attachment):
            if hasattr(attachment, 'filename'):
                attachment_key = '%s/%s' % (str(id), attachment.filename)
                #LOG('SqSQL', INFO, 'addPosting: attachment has %s' % dir(attachment))
                LOG('SqSQL', INFO, 'addPosting: attachment headers are %s' % \
                    attachment.headers.keys())
                #ctype = attachment.headers['content-type']
                ctype = 'unknown'
            else:
                attachment_key = '%s/%s' % (str(id), attachment._name)
                ctype = attachment._ctype
            self.attachments[attachment_key] = attachment
        else:
            attachment_key = None
            ctype = None

        #LOG('SqSQL', INFO, 'addPosting: id = %s' % `id`)
        self.sqlAddPosting(id=int(id), title=title, body=body, subject=subject,
                           created=created, modified=modified, reviewed=reviewed,
                           notify=notify, encoding=encoding, article=article,
                           member=member, author=author, email=email, summary=summary,
                           dept=dept, parent=parent, root=root, level=level,
                           reply_cnt=reply_cnt,
                           attachment_key=attachment_key, attachment_ctype=ctype,
                           keywords=keywords, clients=clients)

        try:
            self.call_cache_invalidate('postings')
        except:
            pass

    security.declareProtected(ManagePortal, 'addCategories')
    def addCategories(self, categories):
        """ Add a list of keyword categories to the database."""
        self.sqlAddCategories(categories=categories)

    security.declareProtected(ManagePortal, 'addKeyword')
    def addKeyword(self, keyword, category):
        """ Add a keyword to the database."""
        self.sqlAddKeyword(keyword=keyword, category=category)

    security.declareProtected(ManagePortal, 'addClients')
    def addClients(self, clients):
        """ Add a list of syndication clients to the database."""
        self.sqlAddClients(clients=clients)

    security.declarePublic('applyFilter')
    def applyFilter(self, filter, get_overflow=0, query_dict = None):
        """Query via a PostingFilter.
        Uses one of three different ZSQLMethods depending on the query.

        The keyword arguments are used by the previous articles display
        code:
            get_overflow - Gets the postings for the current 'day' that
                           exceed filter.max_items.
            query_dict   - Allows code to retrieve the filter query,
                           change it and then submit the changed query
                           along with the filter (for max_items, etc.).
        """

        def do_query(self, query, query_dict):
            if query_dict.get('syndicated_to', []) and query_dict['syndicated_to'][0]:
                #LOG('SqSQL', INFO, 'Calling SqlFilterSyndication("%s", "%s")' % \
                #    (query, query_dict['syndicated_to'][0]))
                results = self.sqlFilterSyndication(filter_query=query,
                                                    syndicated_to=query_dict['syndicated_to'][0])
            elif query_dict.get('keywords', []) and query_dict['keywords'][0]:
                #LOG('SqSQL', INFO, 'Calling SqlFilterKeywords("%s", "%s")' % \
                #    (query, `query_dict['keywords']`))
                results = self.sqlFilterKeywords(filter_query=query,
                                                 keywords=query_dict['keywords'])
            else:
                #LOG('SqSQL', INFO, 'Calling SqlFilterSimple("%s")' % query)
                results = self.sqlFilterSimple(filter_query=query)
            return results

        if not query_dict:
            query_dict = filter.buildQuery()
        query = 'article AND '
        if query_dict.get('reviewed', 0):
            query = query + 'reviewed AND '
        if query_dict.has_key('date'):
            date = query_dict['date']
            date_usage = query_dict['date_usage']
            date_clause_start = len(query)
            if date_usage == 'range:min':
                query = query + "(created >= '%s') " % date.aCommon()
            elif date_usage == 'range:min:max':
                query = query + "(created BETWEEN '%s' AND '%s') " % (date[0].aCommon(),
                                                                  date[1].aCommon())
            date_clause_end = len(query)

        if query_dict.has_key('sort_on'):
            sort_key = query_dict['sort_on']
            if sort_key == 'date':
                # Convert from Squishdot attribute name to SQL column name
                sort_key = 'created'
            query = query + 'ORDER BY %s ' % sort_key
            if query_dict.has_key('sort_order') and query_dict['sort_order'] == 'descending':
                query = query + 'DESC '
            else:
                # Work around a posting filter bug for now
                query = query + 'DESC '

        if filter.max_items:
            if get_overflow:
                pass
            else:
                query = query + 'LIMIT %d ' % filter.max_items

        results = do_query(self, query, query_dict)
        if len(results) < filter.min_items:
            r = self.sqlLastPostingCreated()[0]
            last_created = r.max
            earlier = last_created - 2
            new_date_query = "(created BETWEEN '%s' AND '%s')" % \
                             (earlier.aCommon(), last_created.aCommon())
            new_query = query[:date_clause_start] + new_date_query + query[date_clause_end:]
            results = do_query(self, new_query, query_dict)

        if get_overflow and filter.max_items:
            overflow_results = []
            for i in range(filter.max_items, len(results)):
                overflow_results.append(results[i])
            results = overflow_results

        return results

    security.declarePublic('getFullPosting')
    def getFullPosting(self, id):
        """Get all of the data for a single posting."""

        if type(id) == type(''):
            id = int(id)
        posting = self.sqlGetPosting(id=id)
        if not posting:
            # raise an error?
            return None
        posting = posting[0] # Extract only result

        replies = map(lambda x: x.childid, self.sqlGetReplyIds(id=id))
        posting.setReplies(replies)

        attachment_ids = map(lambda x: x.attachmentid, self.sqlGetAttachmentIds(id=id))
        attachments = map(lambda x, attmap=self.attachments: (x, attmap[x]), attachment_ids)
        posting.setAttachments(attachments)

        root = posting.root
        if not root: root = posting.id

        keywords = map(lambda x: x.keyword, self.sqlGetKeywords(id=root))
        posting.setKeywords(keywords)

        clients = map(lambda x: x.client, self.sqlGetSyndicationClients(id=root))
        posting.setSyndicated_to(clients)

        return posting

    security.declarePublic('descItems')
    def descItems(self, posting):
        """Return a list of the replies to the posting."""
        # posting is a PostingBrain
        # other than changes for that, this is ripped straight from Squishdot
        mlist = []
        mstack = Stack()
        if self.getReplies():
            plist = []
            for id in self.getReplies():
                plist.append(id)
            for id in plist:
                mstack.push(id)
            while not mstack.isEmpty():
                item_id = mstack.pop()
                # Might as well get the full posting since we'll need it if
                # reply_cnt is less than (currently) 50.  Later we could
                # make an optimized version for huge reply_cnt if this proves
                # too slow.
                item = self.getFullPosting(item_id)
                mlist.append(item)
                if item.getReplies():
                    plist = []
                    for id in item.getReplies():
                        plist.append(id)
                    for id in plist:
                        mstack.push(id)
        return map(lambda x: x, mlist)

    security.declarePublic('countPostings')
    def countPostings(self, articles=0):
        """Return the number of postings.
        If articles is true return the number of postings that are articles."""
        try:
            results = self.sqlGetPostingsCount(articles=articles)
            return results[0].count
        except AttributeError:
            # postings_tool probably hasn't had it's db connection set up yet.
            return 0

    security.declarePublic('getUnreviewedPostings')
    def getUnreviewedPostings(self):
        """Return a list of all the unreviewed postings."""
        results = self.sqlFilterUnreviewed()
        return results

    security.declareProtected(ManagePortal, 'getAllPostings')
    def getAllPostings(self):
        """Return a list of all the postings."""
        results = self.sqlFilterAllPostings()
        return results

    security.declarePublic('getSubjectPostings')
    def getSubjectPostings(self, subject, comments_too=0):
        """Return a list of all the postings for a given subject."""
        results = self.sqlFilterSubjectPostings(subject=subject,
                                                comments_too=comments_too)
        return results

    security.declarePublic('getKeywordsPostings')
    def getKeywordsPostings(self, keywords, comments_too=0):
        """Return a list of all the postings matching one or more of a list of keywords.
        A single keyword will be transformed into a list."""
        if type(keywords) == type(''):
            keywords = [keywords]
        results = self.sqlFilterKeywordsPostings(keywords=keywords,
                                                 comments_too=comments_too)
        return results

    security.declarePublic('getAuthorPostings')
    def getAuthorPostings(self, author, comments_too=0):
        """Return a list of all the postings by a given author."""
        results = self.sqlFilterAuthorPostings(author=author,
                                               comments_too=comments_too)
        return results

    security.declarePublic('getTitlePostings')
    def getTitlePostings(self, title, comments_too=0):
        """Return a list of all the postings with a given title."""
        results = self.sqlFilterTitlePostings(title=title,
                                              comments_too=comments_too)
        return results

    security.declarePublic('fullTextSearch')
    def fullTextSearch(self, text):
        """Return a list of all the postings with text in title, summary or body."""
        if not text:
            return []
        else:
            results = self.sqlFullTextSearch(text=text)
            return results

    security.declarePublic('displayFullTextSearch')
    def displayFullTextSearch(self, REQUEST):
        """Return a list of all the postings with text in title, summary or body."""
        body = REQUEST.get('body', '')
        results = self.fullTextSearch(body)
        return self.sqShowSearchResults_html(self, REQUEST,
                                             search_results=results,
                                             result_count=len(results))

    def _has_attachment(self, attachment):
        return attachment and hasattr(attachment, 'filename') and attachment.filename

    security.declareProtected(ManagePortal, 'updatePosting')
    def updatePosting(self, id, title, body, subject, created=None, modified=None, reviewed=0,
                      notify=0, encoding='Plain', article=0, member=None, author=None,
                      email=None, summary=None, dept=None, level=0, attachment=None,
                      delete_attachment=0, keywords=[], clients=[],
                      reparent_to=0, make_article=0):
        """Change a posting."""

        # attachment is a file object with attributes:
        #   _name : file name string
        #   _file : the data
        #   _created : raw time value
        #   _modified : raw time value
        #   _ctype : mime type string
        # For now, store attachments in a PersistentMapping named attachments
        old_attachment_key = None
        if self._has_attachment(attachment):
            idstr = str(id)
            for key in self.attachments.keys():
                if key[:len(idstr)] == idstr:
                    old_attachment_key = key
            attachment_key = '%s/%s' % (str(id), attachment.filename)
            self.attachments[attachment_key] = attachment
            ctype = attachment.headers['content-type']
            delete_attachment = 1
        else:
            attachment_key = None
            ctype = None

        if type(created) in [type(''), type(0)]:
            created = DateTime(created)

        if not modified:
            modified=DateTime()
        elif type(modified) in [type(''), type(0)]:
            modified = DateTime(modified)

        self.sqlUpdatePosting(id=id, title=title, body=body, subject=subject,
                              created=created.aCommon(), modified=modified.aCommon(),
                              reviewed=reviewed, notify=notify, encoding=encoding,
                              article=article, member=None, author=author, email=email,
                              summary=summary, dept=dept, level=level,
                              attachment_key=attachment_key, attachment_ctype=ctype,
                              keywords=keywords, clients=clients,
                              reparent_to=reparent_to, make_article=make_article)

        try:
            self.call_cache_invalidate('postings')
        except:
            pass

        if delete_attachment and old_attachment_key:
            del self.attachments[old_attachment_key]

    security.declareProtected(ManagePortal, 'reparentPosting')
    def reparentPosting(self, id, new_parent=0, make_article=0):
        """Move a subtree of replies to a new place."""
        self.sqlReparentPosting(id=id, reparent_to=new_parent, make_article=make_article)
        try:
            self.call_cache_invalidate('postings')
        except:
            pass

    security.declareProtected(ManagePortal, 'deletePosting')
    def deletePosting(self, id):
        """Delete a posting and all of its replies."""
        posting = self.getFullPosting(id)
        if posting:
            attachments = posting.getAttachments()
            if attachments:
                for key, att in attachments:
                    del self.attachments[key]
            self.sqlDeletePosting(id=id)
        else:
            LOG('SqSQL', ERROR, 'deletePosting(%s) does not exist' % `id`)
        try:
            self.call_cache_invalidate('postings')
        except:
            pass

    #
    # Subject management based on our ZClasses
    #
    # ControlCom_Category is a simple ZClass with the following properties:
    #
    # id : string         Of course.  Used in URLs and REQUEST variables on our site.
    # name : string       Same as subject name in Squishdot.
    # email_tag: string   Used in our email list -- not needed by SqSQL
    # description: text   Topic description for topic listing page, etc.
    # icon_name: string   Used construct image URL by dtml code.
    #
    # and one method:
    #
    # image               Returns an IMG tag constructed from the icon_name -- unused.
    #

    # Subjects that are not normally listed with other subjects (e.g. in selection boxes)
    _hidden_subject_ids = ['onlinedir_cat', 'onlinemag_cat', 'proftrade_cat']

    def _subjects_list(self, show_all=0):
        #Return a list of all the subject ZClasses.
        cats = self.aq_inner.aq_parent.categories
        subjects = cats.objectValues("ControlCom_Category")
        if not show_all:
            subjects = filter(lambda x, hidden=self._hidden_subject_ids: x.id not in hidden,
                              subjects)
        return subjects

    security.declarePublic('has_subjects')
    def has_subjects(self, show_all=0):
        """ returns true if site has subject topics defined"""
        return len(self._subjects_list(show_all=show_all))

    security.declarePublic('subject_count')
    def subject_count(self, show_all=0):
        """ gives count of subject topics defined"""
        return self.has_subjects(show_all=show_all)

    security.declarePublic('subjects_list')
    def subjects_list(self, show_all=0):
        """ returns a list of all the subject names"""
        subjects = self._subjects_list(show_all=show_all)
        return map(lambda x: x.name, subjects)

    security.declarePublic('subject_image')
    def subject_image(self, subj):
        """ returns an image url associated with the subject (can be an empty string)"""
        for subject in self._subjects_list(show_all=1):
            if subj == subject.name:
                return subject.icon_name
        return ''

    security.declarePublic('subject_id')
    def subject_id(self, subj):
        """ returns the category ZClass id associated with the subject
        (can be an empty string)
        """
        for subject in self._subjects_list(show_all=1):
            if subj == subject.name:
                return subject.id
        return ''

    #
    # Keyword manager
    #
    security.declareProtected('Add Postings', 'keywords')
    def keywords(self):
        """List of all known keywords."""
        return map(lambda x: x.keyword, self.sqlGetAllKeywords())

    security.declareProtected('Add Postings', 'base_category')
    def base_category(self):
        """
        The name of the first, required keyword category.
        """
        if not hasattr(self, 'base_category_name'):
            self.base_category_name = 'topics'
        return self.base_category_name

    security.declareProtected('Add Postings', 'categories')
    def categories(self):
        """List of all keyword categories."""
        return map(lambda x: x.keycategory, self.sqlGetAllCategories())

    security.declareProtected(ManagePortal, 'addCategory')
    def addCategory(self, category, REQUEST=None):
        """Add a keyword category."""
        self.sqlAddCategories(categories=[category])
        if REQUEST is not None:
            return self.manage_categoryForm(
                self, REQUEST, manage_tabs_message='Category added.')

    security.declareProtected(ManagePortal, 'deleteCategories')
    def deleteCategories(self, categories, REQUEST=None):
        """Delete one or more keyword categories."""
        self.sqlDeleteCategories(categories=categories)
        if REQUEST is not None:
            return self.manage_categoryForm(
                self, REQUEST, manage_tabs_message='Categories deleted.')

    security.declareProtected('Add Postings', 'keywordsIn')
    def keywordsIn(self, category):
        """List of all keywords in category."""
        if type(category) == type([]):
            category = category[0]
        return map(lambda x: x.keyword,
                   self.sqlGetAllKeywordsIn(category=category))

    security.declareProtected(ManagePortal, 'setKeywords')
    def setKeywords(self, category, keywords, REQUEST=None):
        """Set the keywords for category to keywords."""
        categories = [category]
        self.sqlDeleteCategories(categories=categories)
        self.sqlAddCategories(categories=categories)
        for keyword in keywords:
            self.sqlAddKeyword(category=category, keyword=keyword)
        if REQUEST is not None:
            return self.manage_categoryForm(
                self, REQUEST, category=category,
                manage_tabs_message='Keywords in %s changed.' % category)

    security.declareProtected('Add Postings', 'selectKeywordsIn')
    def selectKeywordsIn(self, category, keyword_list):
        """List of keywords that are in both keyword_list and category."""
        if type(category) == type([]):
            category = category[0]
        return map(lambda x: x.keyword,
                   self.sqlGetKeywordsInCategoryAndList(category=category,
                                                        keywords=keyword_list))

    security.declareProtected(ManagePortal, 'syndicationClients')
    def syndicationClients(self):
        """Return the list of possible syndication clients."""
        return map(lambda x: x.client, self.sqlGetAllSyndicationClients())

    security.declareProtected(ManagePortal, 'setSyndicationClients')
    def setSyndicationClients(self, clients, REQUEST=None):
        """Set the list of possible syndication clients to clients."""

        existing_clients = self.syndicationClients()

        # Remove all obsolete clients
        to_del = []
        for client in existing_clients:
            if client not in clients:
                to_del.append(client)
        self.sqlDeleteClients(clients=to_del)

        # Add all new clients
        to_add = []
        for client in clients:
            if client not in existing_clients:
                to_add.append(client)
        self.sqlAddClients(clients=to_add)

        if REQUEST is not None:
            return self.manage_keywordsForm(
                self, REQUEST, manage_tabs_message='Syndication clients set.')

    #
    # Utility functions from Squishdot
    #

    security.declarePublic('site_url')
    def site_url(self):
        """ url of the site main page """
        return self.aq_inner.aq_parent.absolute_url()

    security.declarePublic('dupString')
    def dupString(self, dstr, count):
        """ returns a duplicate of dstr by count times"""
        return dstr * count

    security.declareProtected('Add Postings', 'suggest_author')
    def suggest_author(self):
        """Provide a default value for a posting author."""
        author = ''
        try: author = self.REQUEST.cookies['_suggest_author']
        except: pass
        if author : author = unquote(author)
        return author or None

    security.declareProtected('Add Postings', 'suggest_email')
    def suggest_email(self):
        """Provide a default value for a posting email address."""
        email  = ''
        try: email  = self.REQUEST.cookies['_suggest_email']
        except: pass
        if email  : email  = unquote(email)
        return email  or None

    security.declareProtected('Add Postings', 'suggest_notify')
    def suggest_notify(self):
        """Provide a default value for email reply notification."""
        return 0

    security.declareProtected('Add Postings', 'suggest_title')
    def suggest_title(self, title):
        """ suggested title of reply """
        return (lower(title[:3])=='re:') and title or 'Re: %s' % title
    
#     # protected by 'View' permission
#     def striptags(self,s):
#         # ''' removes char sequences that invalidate RSS syntax'''
#         return tagRegex.sub("",s)

#     # protected by 'View' permission
#     def html2text(self,s):
#         return html2text(s)

#     # protected by 'View' permission
#     def html2safehtml(self, text):
#         return strip(html2safehtml(text,
#                                    self.getProperty('valid_tags',
#                                                     ('b', 'a', 'i', 'br', 'p',
#                                                      'h3', 'ul', 'li', 'font',
#                                                      'br'))))

    default_template = '<td valign="top">' + \
                         '<input type="checkbox" name="%s:list" value="%s" %s></td>' + \
                       '<td valign="top"><strong>%s</strong></td>'
    null_template = '<td></td><td></td>'

    security.declareProtected('Add Postings', 'makeKeyTableHTML')
    def makeKeyTableHTML(self, keywords, current_keywords, varname='keywords', cols=4):
        """Generate a row-major HTML table of controls, one per keyword.
        """

        def stricmp(str1, str2):
            '''Case insensitive string compare'''
            import string
            lstr1 = string.lower(str1)
            lstr2 = string.lower(str2)
            if lstr1 < lstr2: return -1
            elif lstr1 == lstr2: return 0
            else: return 1

        #LOG('Squishdot', INFO, 'Initial keys are %s' % `keywords`)

        # We're generating row-major but the table syntax is column-major
        # so fill in a table and generate from its contents.

        # The following is safe when integer division
        # promotes to a different type rather than
        # truncates in the future
        maxrow, ignore = divmod(len(keywords), cols)

        colbase = []
        for rownum in range(maxrow + 1):
            colbase.append('')
        entries = []
        for colnum in range(cols):
            entries.append(colbase[:])

        col = 0
        row = 0
        skeywords = keywords[:]
        skeywords.sort(stricmp)
        for key in skeywords:
            entries[col][row] = key
            row = row + 1
            if row > maxrow:
                row = 0
                col = col + 1
        #LOG('Squishdot', INFO, 'Entries is %s' % `entries`)

        # Finally we can generate the table!

        out = '<table>\n'
        for row in range(maxrow + 1):
            out = out + '<tr>'
            for col in range(cols):
                key = entries[col][row]
                if key:
                  if key in current_keywords:
                      checked = 'checked'
                  else:
                      checked = ''
                  out = out + self.default_template % (varname, key, checked, key)
                else:
                  out = out + self.null_template
            out = out + '</tr>\n'
        out = out + '</table>\n'

        return out

    #
    # Squishdot conversion support
    #
    security.declareProtected(ManagePortal, 'addPostingFromSquishdotPosting')
    def addPostingFromSquishdotPosting(self, posting, parent=None):
        """Used to convert and old Squishdot site to an SqSQL site and to
        keep an old Squishdot site and an SqSQL site in sync.
        """
        if posting.body:
            body = join(posting.body, '\n')
        else:
            body = posting.body
        if posting.meta_type == 'Article':
            article = 1
            if posting.summary:
                summary = join(posting.summary, '\n')
            else:
                summary = posting.summary
            dept = posting.dept
        else:
            article = 0
            summary = None
            dept = None
            if not parent:
                parent = posting.thread[-1]
        root = int(posting.threadRoot().id)
        keywords = posting.keywords()
        clients = posting.syndicated_to()

        # Fixup bogus postings forced by kludge to add new Squishdot postings
        title = posting.title
        if not title: title = 'No Title'
        subject = posting.subject
        if not subject: subject = 'Unknown'
        notify = 0
        if posting.notify: notify = 1
        reviewed = 0
        if posting.reviewed: reviewed = 1
        encoding = posting.encoding
        if not encoding: encoding = 'Plain'
            
        try:
            self.addPosting(id=int(posting.id), title=title, body=body,
                            subject=subject, created=posting.created,
                            modified=posting.modified, reviewed=reviewed,
                            notify=notify, encoding=encoding, article=article,
                            member=None, author=posting.author, email=posting.email,
                            summary=summary, dept=dept, parent=parent, root=root,
                            level=posting.level,
                            reply_cnt=0, # We'll recompute this as we load all the replies
                            attachment=posting.file, keywords=keywords, clients=clients)
            self.ids_added.append(int(posting.id))
        except AssertionError, e:
            LOG('SqSQL', ERROR, 'AssertionError posting: id %s' % `posting.id`)
            pass

    security.declareProtected(ManagePortal, 'fillFromSquishdot')
    def fillFromSquishdot(self, sqsite):
        """
        Copy all the postings from an existing Squishdot site to the database.
        """
        from ZODB.POSException import ConflictError
        kmgr = sqsite.keywordManager()
        categories = kmgr.categories()
        self.addCategories(categories)
        for cat in categories:
            keywords = kmgr.keywordsIn(cat)
            for k in keywords:
                self.addKeyword(k, cat)
        clients = kmgr.syndicationClients()
        self.addClients(clients)

        self.ids_added = []
        max_id = 0
        while 1:
            try:
                articles = sqsite.data_map(sqsite.ids) # data_map returns a list, not a dict!
                break
            except ConflictError:
                LOG('SqSQL', ERROR, 'ConflictError: Retrying sqsite.data_map(sqsite.ids)')
        for art in articles:
            while 1:
                try:
                    self.addPostingFromSquishdotPosting(art)
                    break
                except ConflictError:
                    LOG('SqSQL', ERROR, 'ConflictError: Retrying article add')
            if int(art.id) > max_id: max_id = int(art.id)
            repmax = self.addSquishdotReplies(art)
            if repmax > max_id: max_id = repmax

        # Make postings added in SQL start with an id six months in the future
        max_id = DateTime(max_id) + 180
        max_id = int(max_id)
        self.sqlSetIdCounter(newid=max_id)

    security.declareProtected(ManagePortal, 'addSquishdotReplies')
    def addSquishdotReplies(self, post):
        from ZODB.POSException import ConflictError
        max_id = 0
        while 1:
            try:
                replies = post.data_map(post.ids)
                break
            except ConflictError:
                LOG('SqSQL', ERROR, 'ConflictError: Retrying post.date_map(post.ids)')
        for rep in replies:
            while 1:
                try:
                    self.addPostingFromSquishdotPosting(rep, int(post.id))
                    break
                except ConflictError:
                    LOG('SqSQL', ERROR, 'ConflictError: Retrying reply add')
            if int(rep.id) > max_id: max_id = int(rep.id)
            repmax = self.addSquishdotReplies(rep)
            if repmax > max_id: max_id = repmax
        return max_id

    security.declareProtected(ManagePortal, 'updatePostingFromSquishdotPosting')
    def updatePostingFromSquishdotPosting(self, posting):
        """Used to keep an old Squishdot site and an SqSQL site in sync.
        """
        if posting.body:
            body = join(posting.body, '\n')
        else:
            body = posting.body
        if posting.meta_type == 'Article':
            article = 1
            if posting.summary:
                summary = join(posting.summary, '\n')
            else:
                summary = posting.summary
            dept = posting.dept
        else:
            article = 0
            summary = None
            dept = None
        keywords = posting.keywords()
        clients = posting.syndicated_to()

        attachment = None
        delete_attachment = 0
        old_posting = self.getFullPosting(posting.id)
        if old_posting and old_posting.file != posting.file:
            attachment = posting.file
            delete_attachment = 1

        # Fixup bogus postings forced by kludge to add new Squishdot postings
        title = posting.title
        if not title: title = 'No Title'
        subject = posting.subject
        if not subject: subject = 'Unknown'
        notify = 0
        if posting.notify: notify = 1
        reviewed = 0
        if posting.reviewed: reviewed = 1
        encoding = posting.encoding
        if not encoding: encoding = 'Plain'

        try:
            self.updatePosting(posting.id, title, body, subject,
                               created=posting.created,
                               modified=posting.modified, reviewed=reviewed,
                               notify=notify, encoding=encoding, article=article,
                               author=posting.author, email=posting.email,
                               summary=summary, dept=dept, 
                               attachment=attachment, delete_attachment=delete_attachment,
                               keywords=keywords, clients=clients)
        except AssertionError, e:
            LOG('SqSQL', ERROR, 'AssertionError posting: id %s' % `posting.id`)
            pass

    #
    # SQL interface methods
    #
    security.declarePrivate()
    def delSQLQueries(self):
        sqllist = self.objectIds('Z SQL Method')
        self.manage_delObjects(ids=sqllist)

    security.declarePrivate()
    def addSQLQueries(self):

        # To add a brain to an SQL method insert a call to the
        # method.manage_advanced(max_rows, max_cache, cache_time,
        #                        class_name, class_file)
        # E.G.:
        #   self.sqlAddPosting.manage_advanced(1000, 100, 0, "PostingBrain", "SqSQL.Brains")
        #   (the values for max_rows, max_cache and cache_time above are the defaults)

        sqlAddPosting = SQL(
            'sqlAddPosting',
            'Add an article or reply to the database.',
            self.connection,
            ('id:int title:string body:string subject:string created:string modified:string'
             ' reviewed:boolean notify:boolean encoding:string article:boolean member:string'
             ' author:string email:string summary:string dept:string parent:int root:int'
             ' level:int reply_cnt:int attachment_key:string attachment_ctype:string'
             ' keywords:lines clients:lines'),
            _sqlAddPosting)
        self._setObject('sqlAddPosting', sqlAddPosting)

        sqlUpdatePosting = SQL(
            'sqlUpdatePosting',
            'Update an article or reply to the database.',
            self.connection,
            ('id:int title:string body:string subject:string created:string modified:string'
             ' reviewed:boolean notify:boolean encoding:string article:boolean member:string'
             ' author:string email:string summary:string dept:string'
             ' level:int attachment_key:string attachment_ctype:string'
             ' keywords:lines clients:lines'
             ' reparent_to:int make_article:int'),
            _sqlUpdatePosting)
        self._setObject('sqlUpdatePosting', sqlUpdatePosting)

        sqlReparentPosting = SQL(
            'sqlReparentPosting',
            'Reparent an article or reply to the database.',
            self.connection,
            ('id:int reparent_to:int make_article:int'),
            _sqlReparentPosting)
        self._setObject('sqlReparentPosting', sqlReparentPosting)

        sqlDeletePosting = SQL(
            'sqlDeletePosting',
            'Delete an article or reply to the database.',
            self.connection,
            ('id:int'),
            _sqlDeletePosting)
        self._setObject('sqlDeletePosting', sqlDeletePosting)

        sqlFilterSimple = SQL(
            'sqlFilterSimple',
            'Apply a filter without keywords a syndication client.',
            self.connection,
            ('filter_query:string'),
            _sqlFilterSimple)
        self._setObject('sqlFilterSimple', sqlFilterSimple)
        self.sqlFilterSimple.manage_advanced(1000, 100, 0, "PostingBrain", "SqSQL.Brains")

        sqlFilterSyndication = SQL(
            'sqlFilterSyndication',
            'Apply a filter with a syndication client.',
            self.connection,
            ('filter_query:string syndicated_to:string'),
            _sqlFilterSyndication)
        self._setObject('sqlFilterSyndication', sqlFilterSyndication)
        self.sqlFilterSyndication.manage_advanced(1000, 100, 0, "PostingBrain", "SqSQL.Brains")

        sqlFilterKeywords = SQL(
            'sqlFilterKeywords',
            'Apply a filter with keywords.',
            self.connection,
            ('filter_query:string keywords:list'),
            _sqlFilterKeywords)
        self._setObject('sqlFilterKeywords', sqlFilterKeywords)
        self.sqlFilterKeywords.manage_advanced(1000, 100, 0, "PostingBrain", "SqSQL.Brains")

        sqlFilterUnreviewed = SQL(
            'sqlFilterUnreviewed',
            'Apply a filter without keywords a syndication client.',
            self.connection,
            '',
            _sqlFilterUnreviewed)
        self._setObject('sqlFilterUnreviewed', sqlFilterUnreviewed)
        self.sqlFilterUnreviewed.manage_advanced(1000, 100, 0, "PostingBrain", "SqSQL.Brains")

        sqlFilterAllPostings = SQL(
            'sqlFilterAllPostings',
            'Get all postings (for Postings tab).',
            self.connection,
            '',
            _sqlFilterAllPostings)
        self._setObject('sqlFilterAllPostings', sqlFilterAllPostings)
        self.sqlFilterAllPostings.manage_advanced(1000, 100, 0, "PostingBrain", "SqSQL.Brains")

        sqlFilterSubjectPostings = SQL(
            'sqlFilterSubjectPostings',
            'Get all the postings for a single subject',
            self.connection,
            ('subject:text comments_too:int'),
            _sqlFilterSubjectPostings)
        self._setObject('sqlFilterSubjectPostings', sqlFilterSubjectPostings)
        self.sqlFilterSubjectPostings.manage_advanced(1000, 100, 0, "PostingBrain",
                                                      "SqSQL.Brains")

        sqlFilterKeywordsPostings = SQL(
            'sqlFilterKeywordsPostings',
            'Get all the postings with one or more of a list of keywords set.',
            self.connection,
            ('keywords:lines comments_too:int'),
            _sqlFilterKeywordsPostings)
        self._setObject('sqlFilterKeywordsPostings', sqlFilterKeywordsPostings)
        self.sqlFilterKeywordsPostings.manage_advanced(1000, 100, 0, "PostingBrain",
                                                       "SqSQL.Brains")

        sqlFilterAuthorPostings = SQL(
            'sqlFilterAuthorPostings',
            'Get all the postings by a single author',
            self.connection,
            ('author:text comments_too:int'),
            _sqlFilterAuthorPostings)
        self._setObject('sqlFilterAuthorPostings', sqlFilterAuthorPostings)
        self.sqlFilterAuthorPostings.manage_advanced(1000, 100, 0, "PostingBrain",
                                                      "SqSQL.Brains")

        sqlFilterTitlePostings = SQL(
            'sqlFilterTitlePostings',
            'Get all the postings with a given title',
            self.connection,
            ('title:text comments_too:int'),
            _sqlFilterTitlePostings)
        self._setObject('sqlFilterTitlePostings', sqlFilterTitlePostings)
        self.sqlFilterTitlePostings.manage_advanced(1000, 100, 0, "PostingBrain",
                                                      "SqSQL.Brains")

        sqlGetPosting = SQL(
            'sqlGetPosting',
            'Get the complete postings row for a single posting',
            self.connection,
            ('id:int'),
            _sqlGetPosting)
        self._setObject('sqlGetPosting', sqlGetPosting)
        self.sqlGetPosting.manage_advanced(1000, 100, 0, "PostingBrain", "SqSQL.Brains")

        sqlFullTextSearch = SQL(
            'sqlFullTextSearch',
            'Get all the postings matching a full text search',
            self.connection,
            ('text:text'),
            _sqlFullTextSearch)
        self._setObject('sqlFullTextSearch', sqlFullTextSearch)
        self.sqlFullTextSearch.manage_advanced(1000, 100, 0, "PostingBrain", "SqSQL.Brains")

        sqlGetReplyIds = SQL(
            'sqlGetReplyIds',
            'Get the ids of all the replies to this posting.',
            self.connection,
            ('id:int'),
            _sqlGetReplyIds)
        self._setObject('sqlGetReplyIds', sqlGetReplyIds)

        sqlGetAttachmentIds = SQL(
            'sqlGetAttachmentIds',
            'Get the ids of all the attachments to this posting.',
            self.connection,
            ('id:int'),
            _sqlGetAttachmentIds)
        self._setObject('sqlGetAttachmentIds', sqlGetAttachmentIds)

        sqlGetKeywords = SQL(
            'sqlGetKeywords',
            'Get all the keywords associated with this posting.',
            self.connection,
            ('id:int'),
            _sqlGetKeywords)
        self._setObject('sqlGetKeywords', sqlGetKeywords)

        sqlGetSyndicationClients = SQL(
            'sqlGetSyndicationClients',
            'Get all of the clients this posting should be syndicated to.',
            self.connection,
            ('id:int'),
            _sqlGetSyndicationClients)
        self._setObject('sqlGetSyndicationClients', sqlGetSyndicationClients)

        sqlLastPostingCreated = SQL(
            'sqlLastPostingCreated',
            'Apply a filter without keywords a syndication client.',
            self.connection,
            '',
            _sqlLastPostingCreated)
        self._setObject('sqlLastPostingCreated', sqlLastPostingCreated)

        sqlMaxPostingId = SQL(
            'sqlMaxPostingId',
            'Get the largest current posting id.',
            self.connection,
            '',
            _sqlMaxPostingId)
        self._setObject('sqlMaxPostingId', sqlMaxPostingId)

        sqlGetNewId = SQL(
            'sqlGetNewId',
            'Get a new, unused posting id.',
            self.connection,
            '',
            _sqlGetNewId)
        self._setObject('sqlGetNewId', sqlGetNewId)

        sqlSetIdCounter = SQL(
            'sqlSetIdCounter',
            'Set the counter that creates new posting ids.',
            self.connection,
            ('newid:int'),
            _sqlSetIdCounter)
        self._setObject('sqlSetIdCounter', sqlSetIdCounter)

        sqlGetPostingsCount = SQL(
            'sqlGetPostingsCount',
            'Return the number of postings (only Articles if articles is true).',
            self.connection,
            ('articles:int'),
            _sqlGetPostingsCount)
        self._setObject('sqlGetPostingsCount', sqlGetPostingsCount)

        sqlAddCategories = SQL(
            'sqlAddCategories',
            'Add a list of keyword categories to the database.',
            self.connection,
            ('categories:list'),
            _sqlAddCategories)
        self._setObject('sqlAddCategories', sqlAddCategories)

        sqlAddKeyword = SQL(
            'sqlAddKeyword',
            'Add a keyword to the database.',
            self.connection,
            ('keyword:string category:string'),
            _sqlAddKeyword)
        self._setObject('sqlAddKeyword', sqlAddKeyword)

        sqlAddClients = SQL(
            'sqlAddClients',
            'Add a list of syndication clients to the database.',
            self.connection,
            ('clients:list'),
            _sqlAddClients)
        self._setObject('sqlAddClients', sqlAddClients)

        sqlGetAllKeywords = SQL(
            'sqlGetAllKeywords',
            'Get all the keywords.',
            self.connection,
            '',
            _sqlGetAllKeywords)
        self._setObject('sqlGetAllKeywords', sqlGetAllKeywords)

        sqlGetAllCategories = SQL(
            'sqlGetAllCategories',
            'Get all the keyword categories.',
            self.connection,
            '',
            _sqlGetAllCategories)
        self._setObject('sqlGetAllCategories', sqlGetAllCategories)

        sqlGetAllKeywordsIn = SQL(
            'sqlGetAllKeywordsIn',
            'Get all the keywords in a category.',
            self.connection,
            ('category:string'),
            _sqlGetAllKeywordsIn)
        self._setObject('sqlGetAllKeywordsIn', sqlGetAllKeywordsIn)

        sqlGetKeywordsInCategoryAndList = SQL(
            'sqlGetKeywordsInCategoryAndList',
            'Get all the keywords in a category that are also in a list of keywords.',
            self.connection,
            ('category:string keywords:list'),
            _sqlGetKeywordsInCategoryAndList)
        self._setObject('sqlGetKeywordsInCategoryAndList', sqlGetKeywordsInCategoryAndList)

        sqlGetAllSyndicationClients = SQL(
            'sqlGetAllSyndicationClients',
            'Get all the syndication clients.',
            self.connection,
            '',
            _sqlGetAllSyndicationClients)
        self._setObject('sqlGetAllSyndicationClients', sqlGetAllSyndicationClients)

        sqlDeleteCategories = SQL(
            'sqlDeleteCategories',
            'Delete a keyword category and all keywords in it.',
            self.connection,
            ('categories:list'),
            _sqlDeleteCategories)
        self._setObject('sqlDeleteCategories', sqlDeleteCategories)

        sqlDeleteClients = SQL(
            'sqlDeleteClients',
            'Delete some syndication.',
            self.connection,
            ('clients:list'),
            _sqlDeleteClients)
        self._setObject('sqlDeleteClients', sqlDeleteClients)


#
# ZSQLMethod definitions (loaded by addSQLQueries above)
#
_sqlAddPosting="""
INSERT INTO postings (id, created, modified, reviewed, body, title,
                      notify, encoding, article, subject, member,
                      author, email, summary, dept, parent, root,
                      level, reply_cnt)
VALUES (<dtml-sqlvar id type=int>,
        <dtml-sqlvar created type=string>,
        <dtml-sqlvar modified type=string>,
        CAST(<dtml-sqlvar reviewed type=int> AS BOOL),
        <dtml-sqlvar body type=string optional=1>,
        <dtml-sqlvar title type=nb>,
        CAST(<dtml-sqlvar notify type=int> AS BOOL),
        <dtml-sqlvar encoding type=string>,
        CAST(<dtml-sqlvar article type=int> AS BOOL),
        <dtml-sqlvar subject type=nb>,
        <dtml-sqlvar member type=nb optional=1>,
        <dtml-sqlvar author type=nb optional=1>,
        <dtml-sqlvar email type=nb optional=1>,
        <dtml-sqlvar summary type=nb optional=1>,
        <dtml-sqlvar dept type=nb optional=1>,
        <dtml-sqlvar parent type=int optional=1>,
        <dtml-sqlvar root type=int>,
        <dtml-sqlvar level type=int>,
        <dtml-sqlvar reply_cnt type=int>
        );

<dtml-if parent>
  SELECT change_posting_reply(<dtml-sqlvar parent type=int>,
                              <dtml-sqlvar id type=int>);
  SELECT modify_reply_cnt(<dtml-sqlvar parent type=int>, 1);
</dtml-if>

<dtml-if attachment_key>
  INSERT INTO attachments (attachment_key, ctype)
  VALUES (<dtml-sqlvar attachment_key type=string>,
          <dtml-sqlvar attachment_ctype type=string>);

  INSERT INTO posting_attachments (postingid, attachmentid)
  SELECT <dtml-sqlvar id type=int>, attachmentid
  FROM attachments
  WHERE attachment_key = <dtml-sqlvar attachment_key type=string>;
</dtml-if>

<dtml-if keywords>
  SELECT update_keywords(<dtml-sqlvar id type=int>,
  '{<dtml-in keywords>
    "<dtml-var sequence-item>"<dtml-unless sequence-end>, </dtml-unless>
  </dtml-in>}',
  <dtml-var expr="_.len(keywords)">, TRUE);
</dtml-if>

<dtml-if clients>
  SELECT update_syndication(<dtml-sqlvar id type=int>,
  '{<dtml-in clients>
    "<dtml-var sequence-item>"<dtml-unless sequence-end>, </dtml-unless>
  </dtml-in>}',
  <dtml-var expr="_.len(clients)">, TRUE);
</dtml-if>
"""

_sqlUpdatePosting="""
UPDATE postings
SET created = <dtml-sqlvar created type=string>,
    modified = <dtml-sqlvar modified type=string>,
    reviewed = CAST(<dtml-sqlvar reviewed type=int> AS BOOL),
    body = <dtml-sqlvar body type=string optional=1>,
    title = <dtml-sqlvar title type=nb>,
    notify = CAST(<dtml-sqlvar notify type=int> AS BOOL),
    encoding = <dtml-sqlvar encoding type=string>,
    article = CAST(<dtml-sqlvar article type=int> AS BOOL),
    subject = <dtml-sqlvar subject type=nb>,
    member = <dtml-sqlvar member type=nb optional=1>,
    author = <dtml-sqlvar author type=nb optional=1>,
    email = <dtml-sqlvar email type=nb optional=1>,
    summary = <dtml-sqlvar summary type=nb optional=1>,
    dept = <dtml-sqlvar dept type=nb optional=1>,
    level = <dtml-sqlvar level type=int>
WHERE postings.id = <dtml-sqlvar id type=int>;

<dtml-if expr="reparent_to or make_article">
  SELECT reparent_posting(<dtml-sqlvar id type=int>,
                          <dtml-sqlvar reparent_to type=int>,
                          CAST(<dtml-sqlvar make_article type=int> AS BOOL));
</dtml-if>

<dtml-if attachment_key>
  SELECT change_posting_attachment(<dtml-sqlvar id type=int>,
                                   <dtml-sqlvar attachment_key type=string>,
                                   <dtml-sqlvar attachment_ctype type=string>);
</dtml-if>

<dtml-if keywords>
  SELECT update_keywords(<dtml-sqlvar id type=int>,
  '{<dtml-in keywords>
    "<dtml-var sequence-item>"<dtml-unless sequence-end>, </dtml-unless>
  </dtml-in>}',
  <dtml-var expr="_.len(keywords)">, TRUE);
</dtml-if>

<dtml-if clients>
  SELECT update_syndication(<dtml-sqlvar id type=int>,
  '{<dtml-in clients>
    "<dtml-var sequence-item>"<dtml-unless sequence-end>, </dtml-unless>
  </dtml-in>}',
  <dtml-var expr="_.len(clients)">, TRUE);
</dtml-if>

"""

_sqlReparentPosting="""
SELECT reparent_posting(<dtml-sqlvar id type=int>,
                        <dtml-sqlvar reparent_to type=int>,
                        CAST(<dtml-sqlvar make_thread type=int>) AS BOOL);
"""

_sqlDeletePosting="""
SELECT delete_posting(<dtml-sqlvar id type=int>, 'f');
"""

# The list of columns returned by all "filter" SQL queries.  These are
# the minimum required attributes of all instances of PostingBrain.
_filterResultColumns = """
id, created, modified, title, subject, author, email, summary, dept, reply_cnt, encoding,
article, reviewed """

_sqlFilterSimple = """
SELECT %s
FROM postings
WHERE <dtml-var filter_query>
""" % _filterResultColumns

# This depends on filter_query not needing to be parenthesized.  This is required
# because filter_query may end with an ORDER BY clause that must be outside any
# parentheses.
_sqlFilterSyndication = """
SELECT %s
FROM postings, posting_syndication, clients
WHERE posting_syndication.clientid = clients.clientid
  AND clients.client = <dtml-sqlvar syndicated_to type=string>
  AND postingid = postings.id
  AND <dtml-var filter_query>
""" % _filterResultColumns

# This depends on filter_query not needing to be parenthesized.  This is required
# because filter_query may end with an ORDER BY clause that must be outside any
# parentheses.
_sqlFilterKeywords = """
SELECT %s
FROM postings
WHERE keywordid IN (<dtml-in keywords><dtml-unless sequence-start>,</dtml-unless>
                      <dtml-sqlvar sequence-item type=string>
                    </dtml-in>)
  AND postingid = postings.id
  AND <dtml-var filter_query>
""" % _filterResultColumns

_sqlFilterUnreviewed = """
SELECT %s
FROM postings
WHERE reviewed = 'f'
ORDER BY id;
""" % _filterResultColumns

_sqlFilterAllPostings="""
SELECT %s
FROM postings
ORDER BY id DESC;
""" % _filterResultColumns

_sqlFilterSubjectPostings="""
SELECT %s FROM postings
WHERE subject = <dtml-sqlvar subject type=nb> AND reviewed
      <dtml-unless comments_too> AND article </dtml-unless>
ORDER BY created;
""" % _filterResultColumns

_sqlFilterKeywordPostings="""
SELECT %s FROM postings, posting_keywords, keywords 
WHERE keywords.keyword = <dtml-sqlvar keyword type=nb>
   AND posting_keywords.keywordid = keywords.keywordid
   AND postings.id = posting_keywords.postingid
   AND reviewed 
   <dtml-unless comments_too> AND article </dtml-unless>
ORDER BY created;
""" % _filterResultColumns

_sqlFilterKeywordsPostings="""
SELECT %s FROM postings, posting_keywords, keywords 
WHERE keywords.keyword 
   IN (<dtml-in keywords><dtml-sqlvar sequence-item type=nb><dtml-unless sequence-end>,
       </dtml-unless></dtml-in>)
   AND posting_keywords.keywordid = keywords.keywordid
   AND postings.id = posting_keywords.postingid
   AND reviewed 
   <dtml-unless comments_too> AND article </dtml-unless>
ORDER BY created;
""" % _filterResultColumns

_sqlFilterAuthorPostings="""
SELECT %s FROM postings
WHERE author = <dtml-sqlvar author type=nb> AND reviewed
      <dtml-unless comments_too> AND article </dtml-unless>
ORDER BY created;
""" % _filterResultColumns

_sqlFilterTitlePostings="""
SELECT %s FROM postings
WHERE title = <dtml-sqlvar title type=nb> AND reviewed
      <dtml-unless comments_too> AND article </dtml-unless>
ORDER BY created;
""" % _filterResultColumns

# The following SQL queries return PostingBrain instances with an
# attribute for all of the columns in the table 'postings'.

_sqlGetPosting = """
SELECT * FROM postings WHERE id = <dtml-sqlvar id type=int>;
"""

# The following is a quick kludge at full text search that will run
# slowly.  It will be replaced with a real FTS add-on to Postgresql
# someday.
_sqlFullTextSearch="""
SELECT * FROM postings
WHERE (title ~ <dtml-sqlvar text type=string>)
   OR (summary ~ <dtml-sqlvar text type=string>)
   OR (body ~ <dtml-sqlvar text type=string>)
ORDER BY created DESC;
"""

# SQL code to access the postings relation tables.

_sqlGetReplyIds = """
SELECT childid
FROM posting_replies
WHERE postingid = <dtml-sqlvar id type=int>;
"""

_sqlGetAttachmentIds = """
SELECT attachmentid
FROM posting_attachments
WHERE postingid = <dtml-sqlvar id type=int>;
"""

_sqlGetKeywords = """
SELECT keyword
FROM posting_keywords, keywords
WHERE postingid = <dtml-sqlvar id type=int>
  AND posting_keywords.keywordid = keywords.keywordid;
"""

_sqlGetSyndicationClients = """
SELECT client
FROM posting_syndication, clients
WHERE postingid = <dtml-sqlvar id type=int>
  AND posting_syndication.clientid = clients.clientid;
"""

# Misc. utility SQL code related to postings.

_sqlLastPostingCreated = """
SELECT max(created) from postings WHERE article AND reviewed;
"""

_sqlMaxPostingId = """
SELECT max(id) from postings;
"""

_sqlGetNewId = """
SELECT nextval('postings_id_seq');
"""

_sqlSetIdCounter = """
SELECT setval('postings_id_seq', <dtml-sqlvar newid type=int>);
"""

_sqlGetPostingsCount="""
SELECT COUNT(*) from Postings
<dtml-if articles>
  WHERE article
</dtml-if>;
"""

# SQL code to access and maintain the keyword and syndication client
# tables.

_sqlAddCategories = """
<dtml-in categories>
INSERT INTO keycategories (keycategory)
VALUES (<dtml-sqlvar sequence-item type=string>);
</dtml-in>
"""

_sqlDeleteCategories = """
<dtml-in categories>
  SELECT delete_keywords_in_category(<dtml-sqlvar sequence-item type=nb>);
  DELETE FROM keycategories WHERE keycategory = <dtml-sqlvar sequence type=nb>;
</dtml-in>
"""

_sqlAddKeyword = """
INSERT INTO keywords (keyword)
VALUES (<dtml-sqlvar keyword type=string>);

INSERT INTO keycategory_keywords (keycategoryid, keywordid)
SELECT keycategoryid, keywordid
FROM keycategories, keywords
WHERE keycategory = <dtml-sqlvar category type=string>
  AND keyword = <dtml-sqlvar keyword type=string>;
"""

_sqlGetAllKeywords="""
SELECT keyword
FROM keywords
ORDER BY keyword;
"""

_sqlGetAllCategories="""
SELECT keycategory
FROM keycategories ORDER BY keycategory;
"""

_sqlGetAllKeywordsIn="""
SELECT keyword
FROM keywords, keycategories, keycategory_keywords
WHERE keycategories.keycategory = <dtml-sqlvar category type=string>
  AND keycategories.keycategoryid = keycategory_keywords.keycategoryid
  AND keycategory_keywords.keywordid = keywords.keywordid;
"""

_sqlGetKeywordsInCategoryAndList="""
SELECT keyword
FROM keywords, keycategories, keycategory_keywords
WHERE keycategories.keycategory = <dtml-sqlvar category type=string>
  AND keycategories.keycategoryid = keycategory_keywords.keycategoryid
  AND keycategory_keywords.keywordid = keywords.keywordid
  AND <dtml-sqltest keywords type=string column=keyword op=in multiple>;
"""

_sqlGetAllSyndicationClients="""
SELECT client
FROM clients
ORDER BY client;
"""

_sqlAddClients = """
<dtml-in clients>
INSERT INTO clients (client)
VALUES (<dtml-sqlvar sequence-item type=string>);
</dtml-in>
"""

_sqlDeleteClients = """
<dtml-in clients>
  DELETE FROM clients WHERE client = <dtml-sqlvar sequence-item type=nb>;
</dtml-in>
"""

InitializeClass(PostingsTool)
