##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) 2000, 2001, 2002 Control.com, Inc.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################

try:
    from Interface import *
except:
    def Attribute(name, value): pass
    class Base: ' '

class postings_tool(Base):
    """
    Interface for postings_tool, a CMF tool that provides Squishdot-like
    access to postings stored in a PostgreSQL database.

    PostingsTool implements this interface.
    """

    id = Attribute('id', 'Must be set to "postings_tool".')

    #security.declareProtected(ManagePortal, 'newPosting')
    def newPosting(self, title, body, subject, parent_id=0, reviewed=0, notify=0,
                   encoding='Plain', member=None, author=None, email=None, summary=None,
                   dept=None, attachment=None, keywords=[], clients=[]):
        """
        Add a new posting to the database and return it's id.
        """

    #security.declareProtected(ManagePortal, 'addPosting')
    def addPosting(self, id, title, body, subject, created=None, modified=None, reviewed=0,
                   notify=0, encoding='Plain', article=0, member=None, author=None,
                   email=None, summary=None, dept=None, parent=None, root=None,
                   level=0, reply_cnt=0, attachment=None, keywords=[], clients=[]):
        """
        Add an article or reply to the database.
        """

    #security.declareProtected(ManagePortal, 'updatePosting')
    def updatePosting(self, id, title, body, subject, created=None, modified=None, reviewed=0,
                      notify=0, encoding='Plain', article=0, member=None, author=None,
                      email=None, summary=None, dept=None, level=0, attachment=None,
                      delete_attachment=0, keywords=[], clients=[],
                      reparent_to=0, make_article=0):
        """
        Change a posting.
        """

    #security.declareProtected(ManagePortal, 'reparentPosting')
    def reparentPosting(self, id, new_parent=0, make_article=0):
        """
        Move a subtree of replies to a new place.
        """

    #security.declareProtected(ManagePortal, 'deletePosting')
    def deletePosting(self, id):
        """
        Delete a posting and all of its replies.
        """

    #security.declarePublic('applyFilter')
    def applyFilter(self, filter, get_overflow=0, query_dict = None):
        """
        Query via a PostingFilter.
        Uses one of three different ZSQLMethods depending on the query.

        The keyword arguments are used by the previous articles display
        code:
            get_overflow - Gets the postings for the current 'day' that
                           exceed filter.max_items.
            query_dict   - Allows code to retrieve the filter query,
                           change it and then submit the changed query
                           along with the filter (for max_items, etc.).

        Like all search methods, this returns a list of PostingBrain
        instances which have the following attributes:

            id, created, modified, title, subject, author, email, summary,
            dept, reply_cnt, encoding, article, reviewed

        Call 'getFullPosting' to get a PostingBrain with the remaining
        attributes as well (e.g. body).
        """

    #security.declarePublic('getUnreviewedPostings')
    def getUnreviewedPostings(self):
        """
        Return a list of all the unreviewed postings.

        Like all search methods, this returns a list of PostingBrain
        instances which have the following attributes:

            id, created, modified, title, subject, author, email, summary,
            dept, reply_cnt, encoding, article, reviewed

        Call 'getFullPosting' to get a PostingBrain with the remaining
        attributes as well (e.g. body).
        """

    #security.declareProtected(ManagePortal, 'getAllPostings')
    def getAllPostings(self):
        """
        Return a list of all the postings.

        Like all search methods, this returns a list of PostingBrain
        instances which have the following attributes:

            id, created, modified, title, subject, author, email, summary,
            dept, reply_cnt, encoding, article, reviewed

        Call 'getFullPosting' to get a PostingBrain with the remaining
        attributes as well (e.g. body).
        """

    #security.declarePublic('getSubjectPostings')
    def getSubjectPostings(self, subject, comments_too=0):
        """
        Return a list of all the postings for a given subject.

        Like all search methods, this returns a list of PostingBrain
        instances which have the following attributes:

            id, created, modified, title, subject, author, email, summary,
            dept, reply_cnt, encoding, article, reviewed

        Call 'getFullPosting' to get a PostingBrain with the remaining
        attributes as well (e.g. body).
        """

    #security.declarePublic('getKeywordsPostings')
    def getKeywordsPostings(self, keywords, comments_too=0):
        """
        Return a list of all the postings matching one or more of a list of keywords.
        
        A single keyword will be transformed into a list.

        Like all search methods, this returns a list of PostingBrain
        instances which have the following attributes:

            id, created, modified, title, subject, author, email, summary,
            dept, reply_cnt, encoding, article, reviewed

        Call 'getFullPosting' to get a PostingBrain with the remaining
        attributes as well (e.g. body).
        """

    #security.declarePublic('getAuthorPostings')
    def getAuthorPostings(self, author, comments_too=0):
        """
        Return a list of all the postings by a given author.

        Like all search methods, this returns a list of PostingBrain
        instances which have the following attributes:

            id, created, modified, title, subject, author, email, summary,
            dept, reply_cnt, encoding, article, reviewed

        Call 'getFullPosting' to get a PostingBrain with the remaining
        attributes as well (e.g. body).
        """

    #security.declarePublic('getTitlePostings')
    def getTitlePostings(self, title, comments_too=0):
        """
        Return a list of all the postings with a given title.

        Like all search methods, this returns a list of PostingBrain
        instances which have the following attributes:

            id, created, modified, title, subject, author, email, summary,
            dept, reply_cnt, encoding, article, reviewed

        Call 'getFullPosting' to get a PostingBrain with the remaining
        attributes as well (e.g. body).
        """

    #security.declarePublic('fullTextSearch')
    def fullTextSearch(self, text):
        """
        Return a list of all the postings with text in title, summary or body.

        Like all search methods, this returns a list of PostingBrain
        instances which have the following attributes:

            id, created, modified, title, subject, author, email, summary,
            dept, reply_cnt, encoding, article, reviewed

        Call 'getFullPosting' to get a PostingBrain with the remaining
        attributes as well (e.g. body).
        """

    #security.declarePublic('getFullPosting')
    def getFullPosting(self, id):
        """
        Get all of the data for a single posting.

        This includes all of the columns of the main posting table (e.g. 'body')
        and all of the data from the relationship tables.
        """

    #security.declarePublic('descItems')
    def descItems(self, posting):
        """
        Return a list of the replies to the posting.

        The replies will have all of the columns of the postings table (e.g. 'body'),
        which isn't returned in a normal search, but will not have the data from the
        relationship tables.  This means that you will still need to do a 'getFullPosting'
        on a reply if you need this information.
        """

    #
    # Misc. methods
    #

    #security.declarePublic('countPostings')
    def countPostings(self, articles=0):
        """
        Return the number of postings.
        If articles is true return the number of postings that are articles.
        """

    #security.declarePublic('displayFullTextSearch')
    def displayFullTextSearch(self, REQUEST):
        """
        Call fullTextSearch based on REQUEST variables then redirect to the skin
        method 'sqShowSearchResults_html' to display the results.
        """

    #
    # Subject management based on our ZClasses
    #
    # ControlCom_Category is a simple ZClass with the following properties:
    #
    # id : string         Of course.  Used in URLs and REQUEST variables on our site.
    # name : string       Same as subject name in Squishdot.
    # email_tag: string   Used in our email list -- not needed by SqSQL
    # description: text   Topic description for topic listing page, etc.
    # icon_name: string   Used construct image URL by dtml code.
    #
    # and one method:
    #
    # image               Returns an IMG tag constructed from the icon_name -- unused.
    #
    #security.declarePublic('has_subjects')
    def has_subjects(self, show_all=0):
        """
        Returns true if site has subject topics defined
        """

    #security.declarePublic('subject_count')
    def subject_count(self, show_all=0):
        """
        Gives count of subject topics defined
        """

    #security.declarePublic('subjects_list')
    def subjects_list(self, show_all=0):
        """
        Returns a list of all the subject names
        """

    #security.declarePublic('subject_image')
    def subject_image(self, subj):
        """
        Returns an image url associated with the subject (can be an empty string)
        """

    #security.declarePublic('subject_id')
    def subject_id(self, subj):
        """
        Returns the category ZClass id associated with the subject
        (can be an empty string)
        """
    #
    # Keyword manager
    #

    #security.declareProtected('Add Postings', 'keywords')
    def keywords(self):
        """
        List of all known keywords.
        """

    #security.declareProtected('Add Postings', 'keywordsIn')
    def keywordsIn(self, category):
        """
        List of all keywords in category.
        """

    #security.declareProtected('Add Postings', 'base_category')
    def base_category(self):
        """
        The name of the first, required keyword category.
        """

    #security.declareProtected('Add Postings', 'categories')
    def categories(self):
        """
        List of all keyword categories.
        """

    #security.declareProtected(ManagePortal, 'addCategory')
    def addCategory(self, category, REQUEST=None):
        """
        Add a keyword category.
        """

    #security.declareProtected(ManagePortal, 'addCategories')
    def addCategories(self, categories):
        """
        Add a list of keyword categories to the database.
        """

    #security.declareProtected(ManagePortal, 'deleteCategories')
    def deleteCategories(self, categories, REQUEST=None):
        """
        Delete one or more keyword categories.
        """

    #security.declareProtected(ManagePortal, 'addKeyword')
    def addKeyword(self, keyword, category):
        """
        Add a keyword to the database.
        """

    #security.declareProtected(ManagePortal, 'setKeywords')
    def setKeywords(self, category, keywords, REQUEST=None):
        """
        Set the keywords for category to keywords.
        """

    #security.declareProtected('Add Postings', 'selectKeywordsIn')
    def selectKeywordsIn(self, category, keyword_list):
        """
        List of keywords that are in both keyword_list and category.
        """

    #security.declareProtected(ManagePortal, 'syndicationClients')
    def syndicationClients(self):
        """
        Return the list of possible syndication clients.
        """

    #security.declareProtected(ManagePortal, 'addClients')
    def addClients(self, clients):
        """
        Add a list of syndication clients to the database.
        """

    #security.declareProtected(ManagePortal, 'setSyndicationClients')
    def setSyndicationClients(self, clients, REQUEST=None):
        """
        Set the list of possible syndication clients to clients.
        """

    #
    # Utility functions from Squishdot
    #

    #security.declarePublic('site_url')
    def site_url(self):
        """
        Url of the site main page
        """

    #security.declarePublic('dupString')
    def dupString(self, dstr, count):
        """
        Returns a duplicate of dstr by count times
        """

    #security.declareProtected('Add Postings', 'suggest_author')
    def suggest_author(self):
        """
        Provide a default value for a posting author.
        """

    #security.declareProtected('Add Postings', 'suggest_email')
    def suggest_email(self):
        """
        Provide a default value for a posting email address.
        """

    #security.declareProtected('Add Postings', 'suggest_notify')
    def suggest_notify(self):
        """
        Provide a default value for email reply notification.
        """

    #security.declareProtected('Add Postings', 'suggest_title')
    def suggest_title(self, title):
        """
        Suggested title of reply
        """

#     # protected by 'View' permission
#     def striptags(self,s):
#         ''' removes char sequences that invalidate RSS syntax'''

#     # protected by 'View' permission
#     def html2text(self,s):


#     # protected by 'View' permission
#     def html2safehtml(text):

    #security.declareProtected('Add Postings', 'makeKeyTableHTML')
    def makeKeyTableHTML(self, keywords, current_keywords, varname='keywords', cols=4):
        """
        Generate a row-major HTML table of controls, one per keyword.
        """

    #
    # Squishdot conversion support
    #
    #security.declareProtected(ManagePortal, 'addPostingFromSquishdotPosting')
    def addPostingFromSquishdotPosting(self, posting, parent=None):
        """
        Used to convert and old Squishdot site to an SqSQL site and to
        keep an old Squishdot site and an SqSQL site in sync.
        """

    #security.declareProtected(ManagePortal, 'fillFromSquishdot')
    def fillFromSquishdot(self, sqsite):
        """
        Copy all the postings from an existing Squishdot site to the database.
        """

    #security.declareProtected(ManagePortal, 'updatePostingFromSquishdotPosting')
    def updatePostingFromSquishdotPosting(self, posting):
        """
        Used to keep an old Squishdot site and an SqSQL site in sync.
        """

