/*
 * 
 * Zope Public License (ZPL) Version 1.0
 * -------------------------------------
 * 
 * Copyright (c) 2000, 2001, 2002 Control.com, Inc.
 * 
 * This license has been certified as Open Source(tm).
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1. Redistributions in source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. Digital Creations requests that attribution be given to Zope
 *    in any manner possible. Zope includes a "Powered by Zope"
 *    button that is installed by default. While it is not a license
 *    violation to remove this button, it is requested that the
 *    attribution remain. A significant investment has been put
 *    into Zope, and this effort will continue if the Zope community
 *    continues to grow. This is one way to assure that growth.
 * 
 * 4. All advertising materials and documentation mentioning
 *    features derived from or use of this software must display
 *    the following acknowledgement:
 * 
 *      "This product includes software developed by Digital Creations
 *      for use in the Z Object Publishing Environment
 *      (http://www.zope.org/)."
 * 
 *    In the event that the product being advertised includes an
 *    intact Zope distribution (with copyright and license included)
 *    then this clause is waived.
 * 
 * 5. Names associated with Zope or Digital Creations must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission from Digital Creations.
 * 
 * 6. Modified redistributions of any form whatsoever must retain
 *    the following acknowledgment:
 * 
 *      "This product includes software developed by Digital Creations
 *      for use in the Z Object Publishing Environment
 *      (http://www.zope.org/)."
 * 
 *    Intact (re-)distributions of any official Zope release do not
 *    require an external acknowledgement.
 * 
 * 7. Modifications are encouraged but must be packaged separately as
 *    patches to official Zope releases.  Distributions that do not
 *    clearly separate the patches from the original work must be clearly
 *    labeled as unofficial distributions.  Modifications which do not
 *    carry the name Zope may be packaged in any form, as long as they
 *    conform to all of the clauses above.
 * 
 * 
 * Disclaimer
 * 
 *   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
 *   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
 *   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 * 
 * 
 * This software consists of contributions made by Digital Creations and
 * many individuals on behalf of Digital Creations.  Specific
 * attributions are listed in the accompanying credits file.
 * 
 */

-- Main table

CREATE SEQUENCE postings_id_seq;
CREATE TABLE postings (
	id integer PRIMARY KEY,
	created timestamp NOT NULL,
	modified timestamp NOT NULL,
	reviewed boolean,
	body text, -- the lines of Squishdot body joined by \n
	title text NOT NULL,
	notify boolean,
	encoding text NOT NULL CHECK (encoding in ('Plain', 'HTML', 'STX')),
	article boolean, -- is this an Article (the head of a thread)

	subject text NOT NULL,

	member text, -- member id
	author text, -- use if no member id
	email text,  -- use if no member id

	summary text, -- Article only: the lines of Squishdot summary joined by \n
	dept text,    -- Article only

	parent integer, -- id of parent
	root integer NOT NULL, -- id of thread root

	level integer NOT NULL, -- reply level of posting. Used to display posting tree.
	reply_cnt integer NOT NULL -- compute this?
);
CREATE INDEX postings_created_idx ON postings(created);
CREATE INDEX postings_article_idx ON postings(article);
CREATE INDEX postings_reviewed_idx ON postings(reviewed);
CREATE INDEX postings_subject_idx ON postings(subject);

-- Other data tables

CREATE SEQUENCE attachmentid_seq;
CREATE TABLE attachments (
	attachmentid integer DEFAULT nextval('attachmentid_seq') PRIMARY KEY,
	/* attachment_key is of the form <id>/<file name>,
           It is used both as the key of the PersistentMapping used to store
           attachments in Zope and as the path suffix used for file storage,
           either permanent or for interface with the Postgresql large object
           facility. 

	   ctype is specified in all cases.

           The other columns are only used if the data is being stored in this
           database as large objects. */
	attachment_key text NOT NULL,
	ctype text NOT NULL, -- MIME type
	name text, -- file name
	data OID -- file data
);

CREATE SEQUENCE keywordid_seq;
CREATE TABLE keywords (
	keywordid integer DEFAULT nextval('keywordid_seq') PRIMARY KEY,
	keyword text NOT NULL,
	search_exp text
);
CREATE INDEX keywords_keyword_idx ON keywords(keyword);

CREATE SEQUENCE keycategoryid_seq;
CREATE TABLE keycategories (
	keycategoryid integer DEFAULT nextval('keycategoryid_seq')  PRIMARY KEY,
	keycategory text NOT NULL
);
CREATE INDEX keycategories_keycategory_idx ON keycategories(keycategory);

CREATE SEQUENCE clientid_seq;
CREATE TABLE clients (
	clientid integer DEFAULT nextval('clientid_seq') PRIMARY KEY,
	client text NOT NULL
);
CREATE INDEX clients_client_idx ON clients(client);

-- Relation tables

CREATE TABLE posting_replies (
	postingid integer NOT NULL,
	childid  integer NOT NULL UNIQUE REFERENCES postings
);
CREATE INDEX posting_replies_postingid_idx
 ON posting_replies(postingid);

CREATE TABLE posting_attachments (
	postingid integer NOT NULL,
	attachmentid integer NOT NULL UNIQUE REFERENCES attachments
);
CREATE INDEX posting_attach_postingid_idx
 ON posting_attachments(postingid);

CREATE TABLE posting_keywords (
	postingid integer NOT NULL,
	keywordid integer NOT NULL REFERENCES keywords ON DELETE CASCADE
);
CREATE INDEX posting_keys_postingid_idx
 ON posting_keywords(postingid);
CREATE INDEX posting_keys_keys_idx
 ON posting_keywords(keywordid);

CREATE TABLE posting_syndication (
	postingid integer NOT NULL,
	clientid integer NOT NULL REFERENCES clients ON DELETE CASCADE
);
CREATE INDEX posting_synd_postingid_idx
 ON posting_syndication(postingid);
CREATE INDEX posting_synd_clientid_idx
 ON posting_syndication(clientid);

CREATE TABLE keycategory_keywords (
	keycategoryid integer NOT NULL,
	keywordid integer NOT NULL UNIQUE REFERENCES keywords ON DELETE CASCADE
);
CREATE INDEX keycat_keys_keycategoryid_idx
 ON keycategory_keywords(keycategoryid);
