/*
 * 
 * Zope Public License (ZPL) Version 1.0
 * -------------------------------------
 * 
 * Copyright (c) 2000, 2001, 2002 Control.com, Inc.
 * 
 * This license has been certified as Open Source(tm).
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 * 
 * 1. Redistributions in source code must retain the above copyright
 *    notice, this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions, and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 
 * 3. Digital Creations requests that attribution be given to Zope
 *    in any manner possible. Zope includes a "Powered by Zope"
 *    button that is installed by default. While it is not a license
 *    violation to remove this button, it is requested that the
 *    attribution remain. A significant investment has been put
 *    into Zope, and this effort will continue if the Zope community
 *    continues to grow. This is one way to assure that growth.
 * 
 * 4. All advertising materials and documentation mentioning
 *    features derived from or use of this software must display
 *    the following acknowledgement:
 * 
 *      "This product includes software developed by Digital Creations
 *      for use in the Z Object Publishing Environment
 *      (http://www.zope.org/)."
 * 
 *    In the event that the product being advertised includes an
 *    intact Zope distribution (with copyright and license included)
 *    then this clause is waived.
 * 
 * 5. Names associated with Zope or Digital Creations must not be used to
 *    endorse or promote products derived from this software without
 *    prior written permission from Digital Creations.
 * 
 * 6. Modified redistributions of any form whatsoever must retain
 *    the following acknowledgment:
 * 
 *      "This product includes software developed by Digital Creations
 *      for use in the Z Object Publishing Environment
 *      (http://www.zope.org/)."
 * 
 *    Intact (re-)distributions of any official Zope release do not
 *    require an external acknowledgement.
 * 
 * 7. Modifications are encouraged but must be packaged separately as
 *    patches to official Zope releases.  Distributions that do not
 *    clearly separate the patches from the original work must be clearly
 *    labeled as unofficial distributions.  Modifications which do not
 *    carry the name Zope may be packaged in any form, as long as they
 *    conform to all of the clauses above.
 * 
 * 
 * Disclaimer
 * 
 *   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
 *   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 *   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 *   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
 *   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 *   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 *   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 *   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 *   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 *   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 *   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 *   SUCH DAMAGE.
 * 
 * 
 * This software consists of contributions made by Digital Creations and
 * many individuals on behalf of Digital Creations.  Specific
 * attributions are listed in the accompanying credits file.
 * 
 */

DROP FUNCTION modify_reply_cnt(integer, integer);
CREATE FUNCTION modify_reply_cnt(integer, integer) RETURNS integer AS '
DECLARE
    initial_parent_id ALIAS FOR $1;
    parent_id integer := 0;
    amount ALIAS FOR $2;
    parent integer;
    post postings%ROWTYPE;
BEGIN
    parent_id := initial_parent_id;
    WHILE NOT parent_id ISNULL LOOP
        SELECT INTO post * FROM postings WHERE postings.id = parent_id;
        IF NOT FOUND THEN
            RAISE EXCEPTION ''Posting % not found'', parent_id;
        END IF;
        UPDATE postings SET reply_cnt = reply_cnt + amount WHERE id = parent_id;
        parent_id := post.parent;
    END LOOP;
    RETURN 1;
END;
' LANGUAGE 'plpgsql';

DROP FUNCTION change_posting_reply(integer, integer);
CREATE FUNCTION change_posting_reply(integer, integer) RETURNS integer AS '
DECLARE
    parent_id ALIAS FOR $1;
    child_id ALIAS FOR $2;
    xref posting_replies%ROWTYPE;
BEGIN
    SELECT INTO xref * FROM posting_replies 
           WHERE posting_replies.childid = child_id;

    IF FOUND THEN
	IF xref.postingid = parent_id THEN
            -- No change needed
            RETURN 0;
        ELSE
            RAISE EXCEPTION ''Posting % is already a child of % in attempt to change parent to %'', 
                            child_id, xref.postingid, parent_id;
        END IF;
    ELSE
        INSERT INTO posting_replies VALUES (parent_id, child_id);
        RETURN 1;
    END IF;

END;
' LANGUAGE 'plpgsql';

DROP FUNCTION change_posting_attachment(integer, text, text);
CREATE FUNCTION change_posting_attachment(integer, text, text) RETURNS integer AS '
DECLARE
    id ALIAS FOR $1;
    attachment_key ALIAS FOR $2;
    ctype ALIAS FOR $3;
    xref posting_attachments%ROWTYPE;
    att attachments%ROWTYPE;
    new_attachmentid integer;
BEGIN
    SELECT INTO xref * FROM posting_attachments
           WHERE posting_attachments.postingid = id;

    IF FOUND THEN
        SELECT INTO att * FROM attachments
               WHERE attachments.attachmentid = xref.attachmentid;
        IF FOUND THEN
            IF att.attachment_key = attachment_key THEN
                -- No changes needed
                RETURN 0;
	    ELSE
                IF length(trim(attachment_key)) = 0 THEN
                    -- Delete attachment
                    DELETE FROM posting_attachments
                           WHERE posting_attachments.postingid = id;
                    DELETE FROM attachments
                           WHERE attachments.attachmentid = att.attachmentid;
                ELSE
		    -- Replacement attachment
		    UPDATE attachments 
		    SET attachment_key = attachment_key,
			ctype = ctype
		    WHERE attachments.attachmentid = xref.attachmentid;
                END IF;
            END IF;
        ELSE
            -- Inconsistent tables!
            RAISE EXCEPTION ''Attachmentid % not found in attachments.'', xref.attachmentid;
        END IF;
    ELSE
        -- New attachment
        new_attachmentid := nextval(''attachmentid_seq'');
        INSERT INTO attachments(attachmentid, attachment_key, ctype)
               VALUES(new_attachmentid, attachment_key, ctype);
        INSERT INTO posting_attachments
               VALUES(id, new_attachmentid);
    END IF;
    RETURN 1;
END;
' LANGUAGE 'plpgsql';

DROP FUNCTION reparent_posting(integer, integer, bool);
CREATE FUNCTION reparent_posting(integer, integer, bool) RETURNS integer AS '
DECLARE
    posting_id ALIAS FOR $1;
    new_parent_id ALIAS FOR $2;
    make_article ALIAS FOR $3;
    posting postings%ROWTYPE;
    old_parent_id integer;
    new_parent postings%ROWTYPE;
BEGIN
    SELECT INTO posting * FROM postings
           WHERE postings.id = posting_id;
    IF posting.parent <> 0 THEN
        PERFORM modify_reply_cnt(posting.parent, -posting.reply_cnt - 1);
    END IF;
    IF make_article THEN
        UPDATE postings 
               SET root = posting.id,
                   parent = 0,
                   article = ''t''
                   level = 0
               WHERE postings.id = posting_id;
    ELSE -- Move to new parent
        SELECT INTO new_parent * FROM postings
               WHERE postings.id = new_parent_id;
        UPDATE postings
               SET root = new_parent.root,
                   parent = new_parent_id,
                   article = ''f''
                   level = new_parent.level + 1
               WHERE postings.id = posting_id;
        PERFORM update_child_levels(posting_id);
        PERFORM modify_reply_cnt(new_parent_id, posting.reply_cnt + 1);
    END IF;
    RETURN 1;
END;
' LANGUAGE 'plpgsql';

DROP FUNCTION update_child_levels(integer);
CREATE FUNCTION update_child_levels(integer) RETURNS integer AS '
DECLARE
  posting_id ALIAS FOR $1;
  posting postings%ROWTYPE;
  childrow posting_replies%ROWTYPE;
BEGIN
  SELECT INTO posting * FROM postings 
         WHERE postings.id = posting_id;

  FOR childrow IN SELECT * FROM posting_replies 
                           WHERE posting_replies.postingid = posting_id LOOP
    UPDATE postings
      SET level = posting.level + 1
      WHERE postings.id = childrow.childid;
    PERFORM update_child_levels(childrow.childid);
  END LOOP;
  RETURN 1;
END;
' LANGUAGE 'plpgsql';

DROP FUNCTION delete_posting(integer, bool);
CREATE FUNCTION delete_posting(integer, bool) RETURNS integer AS '
DECLARE
  posting_id ALIAS FOR $1;
  -- The purpose of child_call is to save the expense of repeatedly
  -- updating parent reply_cnt as each child is deleted.
  child_call ALIAS FOR $2;
  posting postings%ROWTYPE;
  childrow posting_replies%ROWTYPE;
  attachrow posting_attachments%ROWTYPE;
BEGIN
  -- Delete all child postings
  SELECT INTO posting * FROM postings 
         WHERE postings.id = posting_id;

  FOR childrow IN SELECT * FROM posting_replies 
                           WHERE posting_replies.postingid = posting_id LOOP
    DELETE FROM posting_replies 
           WHERE posting_replies.childid = childrow.childid;
    PERFORM delete_posting(childrow.childid, ''t'');
  END LOOP;

  -- Delete any attachments
  FOR attachrow IN SELECT * FROM posting_attachments 
                            WHERE posting_attachments.postingid = posting_id LOOP
    DELETE FROM posting_attachments 
           WHERE posting_attachments.attachmentid = attachrow.attachmentid;
    DELETE FROM attachments 
           WHERE attachments.attachmentid = attachrow.attachmentid;
  END LOOP;

  DELETE FROM posting_keywords WHERE posting_keywords.postingid = posting_id;
  DELETE FROM posting_syndication WHERE posting_syndication.postingid = posting_id;

  IF posting.parent THEN
      -- Combining these two tests with AND causes a parse error.
      IF NOT child_call THEN
          PERFORM modify_reply_cnt(posting.parent, -posting.reply_cnt - 1);
          DELETE FROM posting_replies WHERE posting_replies.childid = posting_id;
      END IF;
  END IF;
  DELETE FROM postings WHERE postings.id = posting_id;

  RETURN 1;
END;
' LANGUAGE 'plpgsql';

DROP FUNCTION delete_keywords_in_category(text);
CREATE FUNCTION delete_keywords_in_category(text) RETURNS integer AS '
DECLARE
  category ALIAS FOR $1;
  catrow keycategories%ROWTYPE;
  catkeyrow keycategory_keywords%ROWTYPE;
BEGIN
  SELECT INTO catrow * FROM keycategories WHERE keycategory = category;
  -- Make keywords disappear if their category disappears
  FOR catkeyrow IN SELECT * FROM keycategory_keywords 
    WHERE keycategory_keywords.keycategoryid = catrow.keycategoryid
  LOOP
    DELETE FROM keywords WHERE keywords.keywordid = catkeyrow.keywordid;
  END LOOP;
  -- Dont need to delete from keycategory_keywords or posting_keywords
  -- because ON DELETE CASCADE declarations will do it automatically
  DELETE FROM keycategories WHERE keycategory = category;
  RETURN 1;
END; 
' LANGUAGE 'plpgsql';

DROP FUNCTION update_keywords(integer, text[], integer, bool);
CREATE FUNCTION update_keywords(integer, text[], integer, bool) RETURNS integer AS '
DECLARE
  posting_id ALIAS FOR $1;
  new_keywords ALIAS FOR $2;
  num_keywords ALIAS FOR $3;
  delete_current ALIAS FOR $4;
  posting postings%ROWTYPE;
  root_post postings%ROWTYPE;
  root_id integer := 0;
BEGIN
  SELECT INTO posting * FROM postings WHERE id = posting_id;
  root_id := posting.id;
  SELECT INTO root_post * FROM postings WHERE id = posting.root;
  IF FOUND THEN root_id := root_post.id; END IF;
  IF delete_current
  THEN
    DELETE FROM posting_keywords WHERE postingid = root_id;
  END IF;
  FOR i IN 1..num_keywords LOOP
    INSERT INTO posting_keywords (postingid, keywordid)
        SELECT root_id, keywords.keywordid
        FROM keywords
        WHERE keyword = new_keywords[i];
  END LOOP;
  RETURN 1;
END;
' LANGUAGE 'plpgsql';

DROP FUNCTION update_syndication(integer, text[], integer, bool);
CREATE FUNCTION update_syndication(integer, text[], integer, bool) RETURNS integer AS '
DECLARE
  posting_id ALIAS FOR $1;
  new_clients ALIAS FOR $2;
  num_clients ALIAS FOR $3;
  delete_current ALIAS FOR $4;
  posting postings%ROWTYPE;
  root_post postings%ROWTYPE;
  root_id integer := 0;
BEGIN
  SELECT INTO posting * FROM postings WHERE id = posting_id;
  root_id := posting.id;
  SELECT INTO root_post * FROM postings WHERE id = posting.root;
  IF FOUND THEN root_id := root_post.id; END IF;
  IF delete_current
  THEN
    DELETE FROM posting_syndication WHERE postingid = root_id;
  END IF;
  FOR i IN 1..num_clients LOOP
    INSERT INTO posting_syndication (postingid, clientid)
        SELECT root_id, clients.clientid
        FROM clients
        WHERE client = new_clients[i];
  END LOOP;
  RETURN 1;
END;
' LANGUAGE 'plpgsql';

